@@ -2,6 +2,12 @@ import logging
 
 import numpy as np
 import pandas as pd
+from pandas.api.types import is_numeric_dtype
+
+try:
+    from tqdm import tqdm
+except ModuleNotFoundError:
+    tqdm = None
 
 from . import accessor
 
@@ -139,6 +145,10 @@ def get_modellayer_from_screen_depth(ftop, fbot, zvec, left=-999, right=999):
     nan
 
     """
+    if isinstance(zvec, np.ndarray):
+        if np.isnan(zvec).all():
+            logger.warning("vertical datum invalid returning nan")
+            return np.nan
 
     zvec = np.sort(zvec)[::-1]
     ftop_invalid = check_if_var_is_invalid(ftop)
@@ -156,7 +166,7 @@ def get_modellayer_from_screen_depth(ftop, fbot, zvec, left=-999, right=999):
         return lay_fbot
 
     if ftop < fbot:
-        logger.warning("- tube screen top below tube screen bot, switch top and bot")
+        logger.warning("- tube screen top below tube screen bot, switching top and bot")
         fbot, ftop = ftop, fbot
 
     lay_ftop = get_model_layer_z(ftop, zvec, left=left, right=right)
@@ -169,28 +179,26 @@ def get_modellayer_from_screen_depth(ftop, fbot, zvec, left=-999, right=999):
 
     else:
         if lay_fbot == left and lay_ftop == right:
-            logger.debug("- tube screen spans all layers. " "return nan")
+            logger.debug("- tube screen spans all layers. return nan")
             return np.nan
         elif lay_ftop == right:
             logger.debug(
-                "- tube screen top higher than top layer. " f"selected layer {lay_fbot}"
+                f"- tube screen top higher than top layer. selected layer {lay_fbot}"
             )
             return lay_fbot
 
         elif lay_fbot == left:
             logger.debug(
-                "- tube screen bot lower than bottom layer. "
-                f"selected layer {lay_ftop}"
+                f"- tube screen bot lower than bottom layer. selected layer {lay_ftop}"
             )
             return lay_ftop
 
         logger.debug(
-            "- tube screen crosses layer boundary:\n"
-            f"  - layers: {lay_ftop}, {lay_fbot}"
+            f"- tube screen crosses layer boundary:\n  - layers: {lay_ftop}, {lay_fbot}"
         )
 
         logger.debug(
-            f"- tube screen spans {lay_fbot - lay_ftop +1} layers."
+            f"- tube screen spans {lay_fbot - lay_ftop + 1} layers."
             " checking length per layer\n"
             "  - length per layer:"
         )
@@ -205,7 +213,7 @@ def get_modellayer_from_screen_depth(ftop, fbot, zvec, left=-999, right=999):
             else:
                 length_layers[i] = zvec[lay_ftop + i] - zvec[lay_ftop + 1 + i]
 
-            logger.debug(f"    - lay {lay_ftop+i}: {length_layers[i]:.2f}")
+            logger.debug(f"    - lay {lay_ftop + i}: {length_layers[i]:.2f}")
 
         # choose layer with biggest length
         rel_layer = np.argmax(length_layers)
@@ -213,8 +221,6 @@ def get_modellayer_from_screen_depth(ftop, fbot, zvec, left=-999, right=999):
         logger.debug(f"  - selected layer: {lay_out}")
         return lay_out
 
-    return np.nan
-
 
 def get_zvec(x, y, gwf=None, ds=None):
     """get a list with the vertical layer boundaries at a point in the model.
@@ -228,8 +234,8 @@ def get_zvec(x, y, gwf=None, ds=None):
     gwf : flopy.mf6.modflow.mfgwf.ModflowGwf
         modflow model with top and bottoms
     ds : xarray.Dataset
-        xarray Dataset typically create in nlmod. Must have
-        dimensions 'x' and 'y' and variables 'top' and 'botm'.
+        xarray Dataset typically created in nlmod. Must have
+        variables 'top' and 'botm'.
 
     Raises
     ------
@@ -245,7 +251,7 @@ def get_zvec(x, y, gwf=None, ds=None):
     from shapely.geometry import Point
 
     if gwf and not ds:
-        ix = flopy.utils.GridIntersect(gwf.modelgrid)
+        ix = flopy.utils.GridIntersect(gwf.modelgrid, method="vertex")
         if gwf.modelgrid.grid_type == "structured":
             res = ix.intersect(Point(x, y))
             if len(res) > 0:
@@ -273,24 +279,38 @@ def get_zvec(x, y, gwf=None, ds=None):
             )
     elif ds and not gwf:
         # assuming modflow type definition with 1 top and N botms
-        # check extent
-        xmin, xmax, ymin, ymax = ds.attrs["extent"]
-        if (x < xmin) or (x > xmax) or (y < ymin) or (y > ymax):
-            zvec = np.nan
-        elif ds.gridtype == "vertex":
+        # first check if point is within the extent
+        if "angrot" in ds.attrs and ds.attrs["angrot"] != 0.0:
+            import nlmod
+
+            pol = nlmod.dims.get_extent_polygon(ds)
+            if not Point(x, y).within(pol):
+                return np.nan
+        else:
+            xmin, xmax, ymin, ymax = ds.attrs["extent"]
+            if (x < xmin) or (x > xmax) or (y < ymin) or (y > ymax):
+                return np.nan
+
+        if ds.gridtype == "vertex":
             import nlmod
 
             cid = nlmod.dims.xy_to_icell2d((x, y), ds)
             sel = ds.sel(icell2d=cid)
-            zvec = np.concatenate(([sel["top"].data], sel["botm"].data))
+            zvec = np.concatenate(([sel["top"].values], sel["botm"].values))
             mask = np.isnan(zvec)
             idx = np.where(~mask, np.arange(mask.size), 0)
             np.maximum.accumulate(idx, axis=0, out=idx)
             zvec[mask] = zvec[idx[mask]]
         else:
             sel = ds.sel(x=x, y=y, method="nearest")
-            first_notna = np.nonzero(np.isfinite(sel["top"].data))[0][0]
-            zvec = np.concatenate([sel["top"].data[[first_notna]], sel["botm"].data])
+            first_notna = np.nonzero(np.isfinite(np.atleast_1d(sel["top"].values)))[0][
+                0
+            ]
+            if sel["top"].values.shape == tuple():
+                top = np.atleast_1d(sel["top"].values)
+            else:
+                top = np.atleast_1d(sel["top"].values[[first_notna]])
+            zvec = np.concatenate([top, sel["botm"].values])
             mask = np.isnan(zvec)
             idx = np.where(~mask, np.arange(mask.size), 0)
             np.maximum.accumulate(idx, axis=0, out=idx)
@@ -309,17 +329,15 @@ class GwObsAccessor:
     def set_tube_nr(
         self, radius=1, xcol="x", ycol="y", if_exists="error", add_to_meta=False
     ):
-        """This method computes the tube numbers based on the location of the
-        observations.
+        """Set the tube numbers based on the location of the observations.
 
-        Then it sets the value of the tube number:
+        The value of the tube number is set:
 
-        - in the ObsCollection dataframe
-        - as the attribute of an Obs object
-        - in the meta dictionary of the Obs object (only if add_to_meta is
+        - In the ObsCollection dataframe
+        - As the attribute of an Obs object
+        - optionally: In the meta dictionary of the Obs object (only if add_to_meta is
           True)
 
-
         This method is useful for groundwater observations. If two or more
         observation points are close to each other they will be seen as one
         monitoring_well with multiple tubes. The tube_nr is based on the
@@ -352,7 +370,7 @@ class GwObsAccessor:
         # check if column exists in obscollection
         if "tube_nr" in self._obj.columns:
             # set type to numeric
-            if self._obj["tube_nr"].dtype != np.number:
+            if not is_numeric_dtype(self._obj["tube_nr"]):
                 self._obj["tube_nr"] = pd.to_numeric(
                     self._obj["tube_nr"], errors="coerce"
                 )
@@ -388,7 +406,7 @@ class GwObsAccessor:
                             pb_dub, "tube_nr", i + 1, add_to_meta=add_to_meta
                         )
 
-    def set_tube_nr_monitoring_well(
+    def set_tube_nr_location(
         self,
         loc_col,
         radius=1,
@@ -397,32 +415,26 @@ class GwObsAccessor:
         if_exists="error",
         add_to_meta=False,
     ):
-        """This method sets the tube_nr and monitoring_well name of an observation
-        point based on the location of the observations.
-
-        When two or more tubes are close to another, as defined by radius,
-        they are set to the same `monitoring_well` and an increasing `tube_nr` based
-        on depth.
-
-        The value of the tube_nr and the monitoring_well are set:
+        """This method sets the tube_nr and location attributes of a GroundwaterObs
+        based on the location and screen depth of the observations.
 
+        The value of the tube_nr and the location are set:
         - in the ObsCollection dataframe
         - as the attribute of an Obs object
-        - in the meta dictionary of the Obs object (only if add_to_meta is
+        - optional: in the meta dictionary of the Obs object (only if add_to_meta is
           True)
 
-        This method is useful for groundwater observations. If two or more
-        observation points are close to each other they will be seen as one
-        monitoring_well with multiple tubes. The tube_nr is based on the
+        If two or more observations are close to each other they will be seen
+        as one location with multiple tubes. The tube_nr is based on the
         'screen_bottom' attribute of the observations in such a way that
-        the deepest tube has the highest tube number. The monitoring_well is
-        based on the named of the loc_col of the screen with the lowest
+        the deepest tube has the highest tube number. The location name is
+        based on the name in the loc_col of the screen with the lowest
         tube_nr.
 
         Parameters
         ----------
         loc_col : str
-            the column name with the names to use for the monitoring_well
+            the column name with the names to use for the location
         radius : int, optional
             max distance between two observations to be seen as one location,
             by default 1
@@ -445,18 +457,18 @@ class GwObsAccessor:
         """
 
         # check if columns exists in obscollection
-        if "tube_nr" in self._obj.columns or "monitoring_well" in self._obj.columns:
+        if "tube_nr" in self._obj.columns or "location" in self._obj.columns:
             if if_exists == "error":
                 raise RuntimeError(
-                    "the column 'tube_nr or monitoring_well' already exist, set"
+                    "the column 'tube_nr or location' already exist, set"
                     "if_exists='replace' to replace the current values"
                 )
             elif if_exists == "replace":
                 self._obj["tube_nr"] = np.nan
-                self._obj["monitoring_well"] = np.nan
+                self._obj["location"] = np.nan
         else:
             self._obj["tube_nr"] = np.nan
-            self._obj["monitoring_well"] = np.nan
+            self._obj["location"] = np.nan
 
         # apply tube numbers to tubes that are close to eachother
         for name in self._obj.index:
@@ -471,25 +483,25 @@ class GwObsAccessor:
                     self._obj._set_metadata_value(
                         name, "tube_nr", 1, add_to_meta=add_to_meta
                     )
-                    monitoring_well = self._obj.loc[name, loc_col]
+                    location = self._obj.loc[name, loc_col]
                     self._obj._set_metadata_value(
                         name,
-                        "monitoring_well",
-                        monitoring_well,
+                        "location",
+                        location,
                         add_to_meta=add_to_meta,
                     )
                 else:
                     dup_x2 = dup_x.sort_values("screen_bottom", ascending=False)
                     for i, pb_dub in enumerate(dup_x2.index):
                         if i == 0:
-                            monitoring_well = self._obj.loc[pb_dub, loc_col]
+                            location = self._obj.loc[pb_dub, loc_col]
                         self._obj._set_metadata_value(
                             pb_dub, "tube_nr", i + 1, add_to_meta=add_to_meta
                         )
                         self._obj._set_metadata_value(
                             pb_dub,
-                            "monitoring_well",
-                            monitoring_well,
+                            "location",
+                            location,
                             add_to_meta=add_to_meta,
                         )
 
@@ -509,6 +521,12 @@ class GwObsAccessor:
         -------
         pd.Series with the modellayers of each observation
         """
+        msg = (
+            "the get_modellayers method is deprecated, please use the nlmod "
+            "function: nlmod.layer.get_modellayers_screens"
+        )
+        logger.warning(msg)
+
         modellayers = []
         for o in self._obj.obs.values:
             logger.debug("-" * 10 + f"\n {o.name}:")
@@ -532,8 +550,11 @@ class GwObsAccessor:
         -------
         pd.Series with the names of the regis layer of each observation
         """
-
-        return self._obj.obs.apply(lambda o: o.gwobs.get_regis_layer())
+        if tqdm is not None:
+            tqdm.pandas()
+            return self._obj.obs.progress_apply(lambda o: o.gwobs.get_regis_layer())
+        else:
+            return self._obj.obs.apply(lambda o: o.gwobs.get_regis_layer())
 
 
 @accessor.register_obs_accessor("gwobs")
@@ -557,6 +578,9 @@ class GeoAccessorObs:
         int
             modellayer
         """
+        if np.isnan(self._obj.x) or np.isnan(self._obj.y):
+            logger.warning("x or y coordinate is NaN, returning NaN")
+            return np.nan
 
         zvec = get_zvec(self._obj.x, self._obj.y, gwf=gwf, ds=ds)
 
@@ -590,8 +614,8 @@ class GeoAccessorObs:
             return "nan"
 
         # connect to regis netcdf
-        regis_url = r"http://www.dinodata.nl:80/opendap/REGIS/REGIS.nc"
-        regis_ds = xr.open_dataset(regis_url, decode_times=False)
+        regis_url = "https://dinodata.nl/opendap/REGIS/REGIS.nc"
+        regis_ds = xr.open_dataset(regis_url, engine="netcdf4", decode_times=False)
 
         # rename layer in regis netcdf
         regis_ds = regis_ds.rename({"layer": "layer_old"})
