@@ -3,8 +3,11 @@ import os
 
 import matplotlib.pyplot as plt
 import numpy as np
+import pandas as pd
 from matplotlib.gridspec import GridSpec
+from tqdm.auto import tqdm
 
+from ..observation import GroundwaterObs
 from . import accessor
 
 logger = logging.getLogger(__name__)
@@ -27,7 +30,7 @@ class CollectionPlots:
         savedir="figures",
         tmin=None,
         tmax=None,
-        per_monitoring_well=True,
+        per_location=True,
         **kwargs,
     ):
         """Create interactive plots of the observations using bokeh.
@@ -40,11 +43,11 @@ class CollectionPlots:
             start date for timeseries plot
         tmax : dt.datetime, optional
             end date for timeseries plot
-        per_monitoring_well : bool, optional
-            if True plot multiple tubes at the same monitoring_well in one
+        per_location : bool, optional
+            if True plot multiple observations at the same location in one
             figure
         **kwargs :
-            will be passed to the Obs.to_interactive_plot method, options
+            will be passed to the Obs.interactive_plot method, options
             include:
 
             - cols : list of str or None
@@ -55,7 +58,7 @@ class CollectionPlots:
             - hoover_names : list of str
             - plot_colors : list of str
             - ylabel : str
-            - add_filter_to_legend : boolean
+            - add_screen_to_legend : boolean
         """
         _color_cycle = (
             "blue",
@@ -71,26 +74,14 @@ class CollectionPlots:
             "tan",
         )
 
-        # check if observations consist of monitoring wells
-        if per_monitoring_well:
-            otype = self._obj._infer_otype()
-            if isinstance(otype, (list, np.ndarray)):
-                per_monitoring_well = False
-            elif otype.__name__ == "GroundwaterObs":
-                pass
-            else:
-                per_monitoring_well = False
-
-        if per_monitoring_well:
-            plot_names = self._obj.groupby("monitoring_well").count().index
+        if per_location:
+            plot_names = self._obj.groupby("location").count().index
         else:
             plot_names = self._obj.index
 
         for name in plot_names:
-            if per_monitoring_well:
-                oc = self._obj.loc[
-                    self._obj.monitoring_well == name, "obs"
-                ].sort_index()
+            if per_location:
+                oc = self._obj.loc[self._obj.location == name, "obs"].sort_index()
             else:
                 oc = self._obj.loc[[name], "obs"]
 
@@ -106,14 +97,14 @@ class CollectionPlots:
                         p=p,
                         tmin=tmin,
                         tmax=tmax,
-                        plot_colors=[_color_cycle[i + 1]],
+                        plot_colors=[_color_cycle[i]],
                         return_filename=False,
                         **kwargs,
                     )
                     logger.debug(f"created iplot -> {o.name}")
                 except ValueError:
                     logger.error(f"{o.name} has no data between {tmin} and {tmax}")
-                    o.iplot_fname = None
+                    o.meta["iplot_fname"] = None
 
     def interactive_map(
         self,
@@ -121,7 +112,7 @@ class CollectionPlots:
         m=None,
         tiles="OpenStreetMap",
         fname=None,
-        per_monitoring_well=True,
+        per_location=True,
         color="blue",
         legend_name=None,
         add_legend=True,
@@ -130,6 +121,7 @@ class CollectionPlots:
         col_name_lat="lat",
         col_name_lon="lon",
         zoom_start=13,
+        popup_width=620,
         create_interactive_plots=True,
         **kwargs,
     ):
@@ -144,7 +136,7 @@ class CollectionPlots:
           the last one should have add_legend = True to create a correct legend
         - the color of the observation point on the map is now the same color
           as the line of the observation measurements. Also a built-in color
-          cycle is used for different measurements at the same monitoring_well.
+          cycle is used for different measurements at the same location.
 
         Parameters
         ----------
@@ -156,8 +148,8 @@ class CollectionPlots:
             background tiles, default is openstreetmap
         fname : str, optional
             name of the folium map
-        per_monitoring_well : bool, optional
-            if True plot multiple tubes at the same monitoring well in one
+        per_location : bool, optional
+            if True plot multiple observations at the same location in one
             figure
         color : str, optional
             color of the observation points on the map
@@ -166,7 +158,7 @@ class CollectionPlots:
         add_legend : boolean, optional
             add a legend to a plot
         map_label : str, optional
-            add a label to the monitoring wells on the map, the label should be
+            add a label to the observations on the map, the label should be
             1. the attribute of an observation
             2. the key in the meta attribute of the observation
             3. a generic label for each observation in this collection.
@@ -182,9 +174,11 @@ class CollectionPlots:
             start zoom level of the folium ma
         create_interactive_plots : boolean, optional
             if True interactive plots will be created, if False the iplot_fname
-            attribute of the observations is used.
+            in the meta ditctionary of the observations is used.
+        popup_width : int, optional
+            popup width in pixels, optional.
         **kwargs :
-            will be passed to the to_interactive_plots method options are:
+            will be passed to the interactive_plots method options are:
 
             - cols : list of str or None
             - hoover_names : list of str
@@ -194,7 +188,7 @@ class CollectionPlots:
             - hoover_names : list of str
             - plot_colors : list of str
             - ylabel : str
-            - add_filter_to_legend : boolean
+            - add_screen_to_legend : boolean
             - tmin : dt.datetime
             - tmax : dt.datetime
 
@@ -208,10 +202,21 @@ class CollectionPlots:
         import folium
         from folium.features import DivIcon
 
+        # check for empty observations
+        if all([o.empty for o in self._obj.obs.values]):
+            logger.warning("all observations in the collection are empty")
+            for oname in self._obj.index:
+                self._obj._set_metadata_value(
+                    oname, "iplot_fname", None, add_to_meta=True
+                )
+            empty_obs = True
+        else:
+            empty_obs = False
+
         # create interactive bokeh plots
-        if create_interactive_plots:
+        if create_interactive_plots and not empty_obs:
             self._obj.plots.interactive_plots(
-                savedir=plot_dir, per_monitoring_well=per_monitoring_well, **kwargs
+                savedir=plot_dir, per_location=per_location, width=popup_width, **kwargs
             )
 
         # check if observation collection has lat and lon values
@@ -230,7 +235,7 @@ class CollectionPlots:
 
         # create map if no map is given
         if m is None:
-            m = folium.Map([northing, easting], zoom_start=zoom_start)
+            m = folium.Map([northing, easting], zoom_start=zoom_start, tiles=tiles)
 
         # get oc name if no legend name is given
         if legend_name is None:
@@ -240,37 +245,27 @@ class CollectionPlots:
         group_name = '<span style=\\"color: {};\\">{}</span>'.format(color, legend_name)
         group = folium.FeatureGroup(name=group_name)
 
-        # check if observations consist of monitoring wells
-        if per_monitoring_well:
-            otype = self._obj._infer_otype()
-            if isinstance(otype, (list, np.ndarray)):
-                per_monitoring_well = False
-            elif otype.__name__ == "GroundwaterObs":
-                pass
-            else:
-                per_monitoring_well = False
-
-        if per_monitoring_well:
-            plot_names = self._obj.groupby("monitoring_well").count().index
+        if per_location:
+            plot_names = self._obj.groupby("location").count().index
         else:
             plot_names = self._obj.index
 
         for name in plot_names:
-            if per_monitoring_well:
-                oc = self._obj.loc[
-                    self._obj.monitoring_well == name, "obs"
-                ].sort_index()
+            if per_location:
+                oc = self._obj.loc[self._obj.location == name, "obs"].sort_index()
                 o = oc.iloc[-1]
                 name = o.name
             else:
                 o = self._obj.loc[name, "obs"]
 
-            if o.iplot_fname is not None:
-                with open(o.iplot_fname, "r") as f:
+            if o.meta["iplot_fname"] is not None:
+                with open(o.meta["iplot_fname"], "r") as f:
                     bokeh_html = f.read()
 
-                iframe = branca.element.IFrame(html=bokeh_html, width=620, height=420)
-                popup = folium.Popup(iframe, max_width=620)
+                iframe = branca.element.IFrame(
+                    html=bokeh_html, width=popup_width, height=int(popup_width / 1.5)
+                )
+                popup = folium.Popup(iframe, max_width=popup_width)
 
                 folium.CircleMarker(
                     [
@@ -284,7 +279,7 @@ class CollectionPlots:
                 ).add_to(group)
 
                 if map_label != "":
-                    if map_label in o._metadata:
+                    if map_label in o._get_meta_attr():
                         map_label_val = getattr(o, map_label)
                     elif map_label in o.meta.keys():
                         map_label_val = o.meta[map_label]
@@ -304,6 +299,15 @@ class CollectionPlots:
                     ).add_to(group)
             else:
                 logger.info(f"no iplot available for {o.name}")
+                folium.CircleMarker(
+                    [
+                        self._obj.loc[o.name, col_name_lat],
+                        self._obj.loc[o.name, col_name_lon],
+                    ],
+                    icon=folium.Icon(icon="signal"),
+                    fill=True,
+                    color=color,
+                ).add_to(group)
 
         group.add_to(m)
 
@@ -335,6 +339,7 @@ class CollectionPlots:
         fn_save=None,
         check_obs_close_to_screen_bottom=True,
         plot_well_layout_markers=True,
+        plot_obs=True,
     ):
         """Create plot with well layout (left) en observations (right).
 
@@ -366,6 +371,8 @@ class CollectionPlots:
         plot_well_layout_markers : bool, optional
             plots ground level, top tube, screen levels and sandtrap via
             makers. Default is True
+        plot_obs : bool, optional
+            Plots observation. Default is True
 
         TODO:
             - speficy colors via extra column in ObsCollection
@@ -378,6 +385,7 @@ class CollectionPlots:
             - limit y-axis of section plot to observations only
             - remove the checking (if obs are near bottom) from this function
             - moving the legend outside the plot
+            - set xlim of observation plot more tight when tmin is not specified
         """
 
         # prepare column for x location in section plot
@@ -385,18 +393,24 @@ class CollectionPlots:
             # use order in ObsCollection
             plot_x = np.arange(len(self._obj))
         else:
-            plot_x = self._obj[section_colname_x]
+            plot_x = self._obj[section_colname_x].to_numpy()
 
         # create figure
         fig = plt.figure(figsize=(15, 5))
-        gs = GridSpec(
-            1,
-            2,
-            width_ratios=[1, 3],
-        )
-        ax_section = fig.add_subplot(gs[0])
-        ax_obs = fig.add_subplot(gs[1])
-        axes = [ax_section, ax_obs]
+        if plot_obs:
+            # make figure with section plot on left, and righthand plot for obs
+            gs = GridSpec(
+                1,
+                2,
+                width_ratios=[1, 3],
+            )
+            ax_section = fig.add_subplot(gs[0])
+            ax_obs = fig.add_subplot(gs[1])
+            axes = [ax_section, ax_obs]
+        else:
+            # make figure with only one plot on left, observations are not plotted
+            ax_section = fig.add_subplot(111)
+            axes = [ax_section]
 
         if plot_well_layout_markers:
             # plot well layout via markers
@@ -436,44 +450,50 @@ class CollectionPlots:
 
         # loop over all wells, plot observations and details in section plot
         for counter, name in enumerate(self._obj.index):
-            # which column to plot?
-            cols = list(cols)
-            for i, col in enumerate(cols):
-                if col is None:
-                    cols[i] = self._obj.loc[name, "obs"]._get_first_numeric_col_name()
+            if plot_obs:
+                # which column to plot?
+                cols = list(cols)
+                for i, col in enumerate(cols):
+                    if col is None:
+                        cols[i] = self._obj.loc[
+                            name, "obs"
+                        ]._get_first_numeric_col_name()
 
-            # create plot dataframe
-            plot_df = self._obj.loc[name, "obs"][tmin:tmax][cols].copy()
-            if plot_df.empty or plot_df[cols].isna().all().all():
-                logger.warning(f"{name} has no data between {tmin} and {tmax}")
-                continue
+                # create plot dataframe
+                plot_df = self._obj.loc[name, "obs"][tmin:tmax][cols].copy()
+                if plot_df.empty or plot_df[cols].isna().all().all():
+                    logger.warning(f"{name} has no data between {tmin} and {tmax}")
+                    continue
 
-            # PART 1: plot timeseries of observations
-            # one or multiple columns to plot?
-            if len(cols) == 1:
-                p = ax_obs.plot(plot_df, label=name)
+                # PART 1: plot timeseries of observations
+                # one or multiple columns to plot?
+                if len(cols) == 1:
+                    p = ax_obs.plot(plot_df, label=name)
+                else:
+                    for col in cols:
+                        p = ax_obs.plot(plot_df[col], label=f"{name}, {col}")
+                plot_color = p[0].get_color()
+
+                if check_obs_close_to_screen_bottom:
+                    # add horizonal line to plot when minimum observation in first plot
+                    # column is close to bottom of screen
+                    offset = 0.1
+                    if self._obj.loc[name, "screen_bottom"] > (
+                        plot_df[cols[0]].dropna().min() - offset
+                    ):
+                        ax_obs.axhline(
+                            y=self._obj.loc[name, "screen_bottom"],
+                            ls="--",
+                            lw=4,
+                            alpha=0.5,
+                            label=(
+                                f"screen bottom of {name}\n"
+                                "is close to minimum observation"
+                            ),
+                            color=plot_color,
+                        )
             else:
-                for col in cols:
-                    p = ax_obs.plot(plot_df[col], label=f"{name}, {col}")
-
-            if check_obs_close_to_screen_bottom:
-                # add horizonal line to plot when minimum observation in first plot
-                # column is close to bottom of screen
-                offset = 0.1
-                if self._obj.loc[name, "screen_bottom"] > (
-                    plot_df[cols[0]].dropna().min() - offset
-                ):
-                    ax_obs.axhline(
-                        y=self._obj.loc[name, "screen_bottom"],
-                        ls="--",
-                        lw=4,
-                        alpha=0.5,
-                        label=(
-                            f"screen bottom of {name}\n"
-                            "is close to minimum observation"
-                        ),
-                        color=p[0].get_color(),
-                    )
+                plot_color = "hotpink"
 
             # PART 2: fancy section plot with lines along tube
 
@@ -493,7 +513,7 @@ class CollectionPlots:
             ax_section.plot(
                 [plot_x[counter]] * 2,
                 [self._obj.loc[name, "screen_top"], self._obj.loc[name, "tube_top"]],
-                color=p[0].get_color(),
+                color=plot_color,
                 lw=3,
             )
 
@@ -505,43 +525,44 @@ class CollectionPlots:
                         self._obj.loc[name, "screen_bottom"],
                         self._obj.loc[name, "tube_bottom"],
                     ],
-                    color=p[0].get_color(),
+                    color=plot_color,
                     lw=3,
                 )
 
-            # PART 3: fancy section plot with bandwith of observations
-            ax_section.scatter(
-                plot_x[counter],
-                plot_df.quantile(q=0.95),
-                section_markersize,
-                marker=(3, 0, 0),
-                color="blue",
-                alpha=0.5,
-                label="95% observation",
-                zorder=100,
-            )
-            ax_section.scatter(
-                plot_x[counter],
-                plot_df.median(),
-                section_markersize,
-                marker=(4, 0, 90),
-                color="green",
-                alpha=0.5,
-                label="median",
-                zorder=100,
-            )
-            ax_section.scatter(
-                plot_x[counter],
-                plot_df.quantile(q=0.05),
-                section_markersize,
-                marker=(3, 0, 180),
-                color="red",
-                alpha=0.5,
-                label="5% observation",
-                zorder=100,
-            )
+            if plot_obs:
+                # PART 3: fancy section plot with bandwith of observations
+                ax_section.scatter(
+                    plot_x[counter],
+                    plot_df.quantile(q=0.95),
+                    section_markersize,
+                    marker=(3, 0, 0),
+                    color="blue",
+                    alpha=0.5,
+                    label="95% observation",
+                    zorder=100,
+                )
+                ax_section.scatter(
+                    plot_x[counter],
+                    plot_df.median(),
+                    section_markersize,
+                    marker=(4, 0, 90),
+                    color="green",
+                    alpha=0.5,
+                    label="median",
+                    zorder=100,
+                )
+                ax_section.scatter(
+                    plot_x[counter],
+                    plot_df.quantile(q=0.05),
+                    section_markersize,
+                    marker=(3, 0, 180),
+                    color="red",
+                    alpha=0.5,
+                    label="5% observation",
+                    zorder=100,
+                )
 
-        logger.info(f"created sectionplot -> {name}")
+            logger.info(f"created sectionplot -> {name}")
 
         # layout
         if section_label_x is None:
@@ -555,13 +576,16 @@ class CollectionPlots:
         else:
             ax_section.set_xlabel(section_label_x)
 
-        # rotate labels on observation axis
-        ax_obs.set_xticks(
-            ax_obs.get_xticks(),
-            ax_obs.get_xticklabels(),
-            rotation="vertical",
-            fontsize="small",
-        )
+        if plot_obs:
+            ax_obs.set_xlim(left=tmin, right=tmax)
+
+            # rotate labels on observation axis
+            ax_obs.set_xticks(
+                ax_obs.get_xticks(),
+                ax_obs.get_xticklabels(),
+                rotation="vertical",
+                fontsize="small",
+            )
 
         if ylabel == "auto":
             # has collection uniform unit?
@@ -604,6 +628,123 @@ class CollectionPlots:
 
         return fig, axes
 
+    def series_per_group(
+        self,
+        plot_column,
+        by=None,
+        savefig=True,
+        outputdir=".",
+        naming_method=None,
+        units_for_well_screen_depth="mNAP",
+    ):
+        """Plot time series per group.
+
+        The default groupby is based on identical x, y coordinates, so plots unique
+        time series per location.
+
+        Parameters
+        ----------
+        plot_column : str
+            name of column containing time series data
+        by : (list of) str or (list of) array-like
+            groupby parameters, default is None which sets groupby to
+            columns ["x", "y"].
+        savefig : bool, optional
+            save figures, by default True, if False returns axes handles to plots.
+        outputdir : str, optional
+            path to output directory, by default the current directory (".")
+        naming_method : str, optional
+            method to determine file names for plots, default is None, which uses the
+            following template:
+                "series_by_{groupby columns}_group_{groupby values}.png".
+            If a string is passed, it is interpreted as a column name and the filename
+            contains the unique values from that column per group:
+                "series_group_{unique values in column}.png"
+            The final option includes "infer_name_monitoring_well", which attempts to
+            obtain the name of the monitoring well from the final name in the group,
+            by splitting the string on "-" or "_", and using the first part.
+            For example, if the name is "PB001-001", the filename will be "PB001.png".
+        units_for_well_screen_depth : str, optional
+            if the observations are GroundWaterObs, the units of the screen depth are
+            added to the legend. This keyword argument defines the units for the screen
+            depth. Default is 'mNAP'.
+
+        Returns
+        -------
+        axes : list
+            returns list of axes handles if savefig=False
+        """
+        if by is None:
+            by = ["x", "y"]
+        gr = self._obj.groupby(by=by)
+
+        axes = []
+
+        for groupname, group in tqdm(
+            gr, desc="Plotting series per group", total=len(gr)
+        ):
+            f, ax = plt.subplots(1, 1, figsize=(10, 3))
+            for name, row in group.iterrows():
+                if isinstance(row.obs, GroundwaterObs):
+                    if units_for_well_screen_depth == "mNAP":
+                        lbl = (
+                            f"{name} (NAP{row['screen_top']:+.1f}"
+                            f"-{row['screen_bottom']:+.1f} m)"
+                        )
+                    else:
+                        lbl = (
+                            f"{name} ({row['screen_top']:.1f}"
+                            f"-{row['screen_bottom']:.1f} "
+                            f"{units_for_well_screen_depth})"
+                        )
+                else:
+                    lbl = f"{name}"
+                ax.plot(
+                    row.obs.index,
+                    row.obs[plot_column],
+                    label=lbl,
+                )
+            ax.legend(
+                loc=(0, 1),
+                frameon=False,
+                ncol=min(group.index.size, 3),
+                fontsize="x-small",
+            )
+            ax.set_ylabel(row["unit"])
+            ax.grid(True)
+            f.tight_layout()
+
+            if savefig:
+                if isinstance(by, list):
+                    by_name = "-".join(by)
+                    groupname = "-".join(groupname)
+                else:
+                    by_name = by
+                if naming_method is None:
+                    filename = f"series_by_{by_name}_group_{groupname}.png"
+                elif naming_method == "infer_name_monitoring_well":
+                    if "-" in name:
+                        filename = f"{name.split('-')[0]}.png"
+                    elif "_" in name:
+                        filename = f"{name.split('_')[0]}.png"
+                    else:
+                        filename = f"{name}.png"
+                elif isinstance(naming_method, str):
+                    filename = (
+                        f"series_by_{by_name}_group_"
+                        f"{'-'.join(group[naming_method].unique().tolist())}.png"
+                    )
+
+                f.savefig(
+                    os.path.join(outputdir, filename), bbox_inches="tight", dpi=150
+                )
+                plt.close(f)
+            else:
+                axes.append(ax)
+
+        if not savefig:
+            return axes
+
 
 @accessor.register_obs_accessor("plots")
 class ObsPlots:
@@ -620,11 +761,12 @@ class ObsPlots:
         plot_freq=(None,),
         tmin=None,
         tmax=None,
-        hoover_names=("Peil",),
+        hoover_names=(None,),
         hoover_date_format="%Y-%m-%d",
         ylabel=None,
         plot_colors=("blue",),
         add_screen_to_legend=False,
+        width=600,
         return_filename=False,
     ):
         """Create an interactive plot of the observations using bokeh.
@@ -666,6 +808,8 @@ class ObsPlots:
         add_screen_to_legend : boolean, optional
             if True the attributes screen_top and screen_bottom
             are added to the legend name
+        width : int, optional
+            width of the interactive figure.
         return_filename : boolean, optional
             if True filename will be returned
 
@@ -675,14 +819,19 @@ class ObsPlots:
             filename of the bokeh plot or reference to bokeh plot
         """
 
-        from bokeh.models import ColumnDataSource, HoverTool
+        from bokeh.models import ColumnDataSource, HoverTool, Range1d
         from bokeh.plotting import figure, save
         from bokeh.resources import CDN
 
+        xlim = None
+
+        hoover_names = list(hoover_names)
         cols = list(cols)
         for i, col in enumerate(cols):
             if col is None:
                 cols[i] = self._obj._get_first_numeric_col_name()
+            if hoover_names[i] is None:
+                hoover_names[i] = cols[i]
 
         # create plot dataframe
         plot_df = self._obj[tmin:tmax].copy()
@@ -691,10 +840,20 @@ class ObsPlots:
             raise ValueError(
                 "{} has no data between {} and {}".format(self._obj.name, tmin, tmax)
             )
+        elif len(plot_df) == 1:
+            markers = ["circle"] * len(cols)
+            # set xlim because there is only one measurement
+            obsdate = plot_df.index[0]
+            xlim = Range1d(
+                start=obsdate - pd.Timedelta(days=365),
+                end=obsdate + pd.Timedelta(days=365),
+            )
 
         # create plot
         if p is None:
-            p = figure(width=600, height=400, x_axis_type="datetime", title="")
+            p = figure(
+                width=width, height=int(width / 1.5), x_axis_type="datetime", title=""
+            )
             if ylabel is None:
                 ylabel = self._obj.unit
             p.yaxis.axis_label = ylabel
@@ -735,7 +894,6 @@ class ObsPlots:
                 )
 
             # plot data
-
             if markers[i] in ["line", "l"]:
                 plots.append(
                     p.line(
@@ -772,6 +930,10 @@ class ObsPlots:
             hover = HoverTool(renderers=[plots[i]], tooltips=tooltips_p, mode="vline")
             p.add_tools(hover)
 
+        # set xlim
+        if xlim is not None:
+            p.x_range = xlim
+
         p.legend.location = "top_left"
         p.legend.click_policy = "mute"
 
@@ -779,10 +941,12 @@ class ObsPlots:
         if savedir is not None:
             if not os.path.isdir(savedir):
                 os.makedirs(savedir)
-            self._obj.iplot_fname = os.path.join(savedir, self._obj.name + ".html")
-            save(p, self._obj.iplot_fname, resources=CDN, title=self._obj.name)
+            self._obj.meta["iplot_fname"] = os.path.join(
+                savedir, str(self._obj.name) + ".html"
+            )
+            save(p, self._obj.meta["iplot_fname"], resources=CDN, title=self._obj.name)
 
         if return_filename:
-            return self._obj.iplot_fname
+            return self._obj.meta["iplot_fname"]
         else:
             return p
