@@ -1,22 +1,35 @@
 """
 https://www.bro-productomgeving.nl/bpo/latest/informatie-voor-softwareleveranciers/url-s-publieke-rest-services
+
+function levels:
+1. get_obs_list_from_gmn: list of observations from grondwatermonitoring net
+    2. get_bro_groundwater: single observation
+        3. measurements_from_gld:
+        4. get_metadata_from_gmw:
+        5. get_gld_ids_from_gmw:
+
 """
 
 import json
 import logging
 import xml.etree.ElementTree
+from functools import lru_cache
 
 import numpy as np
 import pandas as pd
 import requests
-from pyproj import Transformer
+from pyproj import Proj, Transformer
+from requests.adapters import HTTPAdapter, Retry
 from tqdm import tqdm
 
+from ..rcparams import rcParams
+from ..util import EPSG_28992
+
 logger = logging.getLogger(__name__)
 
 
 def get_obs_list_from_gmn(bro_id, ObsClass, only_metadata=False, keep_all_obs=True):
-    """
+    """get a list of observation from a groundwater monitoring network.
 
     Parameters
     ----------
@@ -52,7 +65,11 @@ def get_obs_list_from_gmn(bro_id, ObsClass, only_metadata=False, keep_all_obs=Tr
     req = requests.get(url)
 
     if req.status_code > 200:
-        print(req.json()["errors"][0]["message"])
+        logger.error(
+            "could not get monitoring wells, this groundwater monitoring net is"
+            "probably too big. Try a smaller extent"
+        )
+        req.raise_for_status()
 
     ns = {"xmlns": "http://www.broservices.nl/xsd/dsgmn/1.0"}
 
@@ -73,7 +90,7 @@ def get_obs_list_from_gmn(bro_id, ObsClass, only_metadata=False, keep_all_obs=Tr
             bro_id=gmw_id, tube_nr=tube_nr, only_metadata=only_metadata
         )
         if o.empty:
-            logger.warning(
+            logger.debug(
                 f"no measurements found for gmw_id {gmw_id} and tube number {tube_nr}"
             )
             if keep_all_obs:
@@ -121,7 +138,7 @@ def get_bro_groundwater(bro_id, tube_nr=None, only_metadata=False, **kwargs):
         metadata.
 
     """
-    logger.info(f"reading bro_id {bro_id}")
+    logger.debug(f"reading bro_id {bro_id}")
 
     if bro_id.startswith("GLD"):
         if only_metadata:
@@ -130,26 +147,34 @@ def get_bro_groundwater(bro_id, tube_nr=None, only_metadata=False, **kwargs):
 
     elif bro_id.startswith("GMW"):
         if tube_nr is None:
-            raise ValueError("if bro_id is GMW a filternumber should be specified")
+            raise ValueError("if bro_id is GMW a tube_nr should be specified")
 
         meta = get_metadata_from_gmw(bro_id, tube_nr)
-        gld_id = get_gld_id_from_gmw(bro_id, tube_nr)
+        gld_ids = get_gld_ids_from_gmw(bro_id, tube_nr)
 
-        if gld_id is None:
+        if gld_ids is None:
             meta["name"] = f"{bro_id}_{tube_nr}"
             only_metadata = True  # cannot get time series without gld id
         else:
-            meta["name"] = gld_id
+            meta["name"] = f"{bro_id}_{tube_nr}"
+            meta["gld_ids"] = gld_ids
 
         if only_metadata:
             empty_df = pd.DataFrame()
             return empty_df, meta
 
-        return measurements_from_gld(gld_id, **kwargs)
+        dfl = []
+        for i, gld_id in enumerate(gld_ids):
+            df, meta_new = measurements_from_gld(gld_id, **kwargs)
+            meta.update(meta_new)
+            dfl.append(df)
+        df = pd.concat(dfl, axis=0).sort_index()
+
+        return df, meta
 
 
-def get_gld_id_from_gmw(bro_id, tube_nr, quality_regime="IMBRO/A"):
-    """get bro_id of a grondwterstandendossier (gld) from a bro_id of a
+def get_gld_ids_from_gmw(bro_id, tube_nr):
+    """get bro_ids of multiple grondwaterstandendossier (gld) from a bro_id of a
     grondwatermonitoringsput (gmw).
 
     Parameters
@@ -158,8 +183,6 @@ def get_gld_id_from_gmw(bro_id, tube_nr, quality_regime="IMBRO/A"):
         starts with 'GLD' or 'GMW' e.g. 'GMW000000036287'.
     tube_nr : int
         tube number.
-    quality_regime : str
-        either choose 'IMBRO/A' or 'IMBRO'.
 
     Raises
     ------
@@ -170,8 +193,8 @@ def get_gld_id_from_gmw(bro_id, tube_nr, quality_regime="IMBRO/A"):
 
     Returns
     -------
-    str
-        bro_id of a grondwaterstandonderzoek (gld).
+    list of str or None
+        bro_ids of a grondwaterstandonderzoek (gld).
 
     """
     if not bro_id.startswith("GMW"):
@@ -181,49 +204,27 @@ def get_gld_id_from_gmw(bro_id, tube_nr, quality_regime="IMBRO/A"):
     req = requests.get(url)
 
     if req.status_code > 200:
-        print(req.json()["errors"][0]["message"])
+        logger.error(f"could not get gld ids {req.text}")
+        req.raise_for_status()
 
     d = json.loads(req.text)
 
     if len(d["monitoringTubeReferences"]) == 0:
-        logger.info(
+        logger.debug(
             f"no groundwater level dossier for {bro_id} and tube number {tube_nr}"
         )
         return None
 
     for tube in d["monitoringTubeReferences"]:
         if tube["tubeNumber"] == tube_nr:
-            if len(tube["gldReferences"]) == 1:
-                return tube["gldReferences"][0]["broId"]
-            elif len(tube["gldReferences"]) == 0:
-                logger.info(
+            if len(tube["gldReferences"]) == 0:
+                logger.debug(
                     f"no groundwater level dossier for {bro_id} and tube number"
                     f"{tube_nr}"
                 )
                 return None
-            elif len(tube["gldReferences"]) == 2:
-                logger.info(
-                    f"two gld references found for GMW {bro_id} and tube nr"
-                    f"{tube_nr}, using {quality_regime} quality regime"
-                )
-                for gldref in tube["gldReferences"]:
-                    url2 = gldref["url"]
-                    req2 = requests.get(url2)
-                    ns = {
-                        "ns11": "http://www.broservices.nl/xsd/dsgld/1.0",
-                        "brocom": "http://www.broservices.nl/xsd/brocommon/3.0",
-                    }
-                    tree = xml.etree.ElementTree.fromstring(req2.text)
-                    gld = tree.findall(".//ns11:GLD_O", ns)[0]
-                    qualityRegime = gld.find("brocom:qualityRegime", ns).text
-                    if qualityRegime == quality_regime:
-                        return gldref["broId"]
-                logger.info(
-                    f"no gld reference with quality regime {quality_regime} was found"
-                )
-                return None
             else:
-                raise RuntimeError("unexpected number of gld references")
+                return [gldref["broId"] for gldref in tube["gldReferences"]]
 
 
 def measurements_from_gld(
@@ -275,10 +276,15 @@ def measurements_from_gld(
     if tmax is not None:
         tmax = pd.to_datetime(tmax)
         params["observationPeriodEndDate"] = tmax.strftime("%Y-%m-%d")
-    req = requests.get(url.format(bro_id), params=params)
+
+    # add some logic to retry in case of a 429 response
+    s = requests.Session()
+    retries = Retry(total=5, backoff_factor=0.5, status_forcelist=[429])
+    s.mount("https://", HTTPAdapter(max_retries=retries))
+    req = s.get(url.format(bro_id), params=params)
 
     if req.status_code > 200:
-        print(req.json()["errors"][0]["message"])
+        req.raise_for_status()
 
     ns = {
         "ns11": "http://www.broservices.nl/xsd/dsgld/1.0",
@@ -295,11 +301,12 @@ def measurements_from_gld(
         raise (Exception("Only one gld supported"))
     gld = glds[0]
 
-    meta = {"name": bro_id, "source": "BRO"}
-    meta["monitoring_well"] = gld.find("ns11:monitoringPoint//gldcommon:broId", ns).text
+    meta = {"source": "BRO"}
+    meta["location"] = gld.find("ns11:monitoringPoint//gldcommon:broId", ns).text
     meta["tube_nr"] = int(
         gld.find("ns11:monitoringPoint//gldcommon:tubeNumber", ns).text
     )
+    meta["name"] = f"{meta['location']}_{meta['tube_nr']}"
     gmn = gld.find("ns11:groundwaterMonitoringNet//gldcommon:broId", ns)
     if gmn is None:
         meta["monitoringsnet"] = None
@@ -317,7 +324,7 @@ def measurements_from_gld(
 
     # to dataframe
     df = pd.DataFrame(
-        index=pd.to_datetime(times),
+        index=pd.to_datetime(times, utc=True).tz_convert("CET"),
         data={"values": values, "qualifier": qualifiers},
     )
 
@@ -325,14 +332,14 @@ def measurements_from_gld(
     if to_wintertime:
         # remove time zone information by transforming to dutch winter time
         df.index = pd.to_datetime(df.index, utc=True).tz_localize(None) + pd.Timedelta(
-            1, unit="H"
+            1, unit="h"
         )
 
     # duplicates
     if df.index.has_duplicates and drop_duplicate_times:
         duplicates = df.index.duplicated(keep="first")
         message = "{} contains {} duplicates (of {}). Keeping only first values."
-        logger.info(message.format(bro_id, duplicates.sum(), len(df)))
+        logger.debug(message.format(bro_id, duplicates.sum(), len(df)))
         df = df[~duplicates]
 
     df = df.sort_index()
@@ -341,7 +348,7 @@ def measurements_from_gld(
     df = df.loc[tmin:tmax]
 
     # add metadata from gmw
-    meta.update(get_metadata_from_gmw(meta["monitoring_well"], meta["tube_nr"]))
+    meta.update(get_metadata_from_gmw(meta["location"], meta["tube_nr"]))
 
     return df, meta
 
@@ -383,7 +390,7 @@ def get_full_metadata_from_gmw(bro_id, tube_nr):
     if len(gmws) != 1:
         raise (Exception("Only one gmw supported"))
     gmw = gmws[0]
-    meta = {"monitoring_well": bro_id, "tube_nr": tube_nr, "source": "BRO"}
+    meta = {"location": bro_id, "tube_nr": tube_nr, "source": "BRO"}
     for child in gmw:
         key = child.tag.split("}", 1)[1]
         if len(child) == 0:
@@ -436,11 +443,95 @@ def get_full_metadata_from_gmw(bro_id, tube_nr):
     return meta
 
 
+@lru_cache()
+def _get_gmw_from_bro_id(bro_id, retries=0):
+    """get a gmw object from a bro_id
+
+    Parameters
+    ----------
+    bro_id : str
+        bro id of groundwater monitoring well e.g. 'GMW000000036287'.
+
+    Returns
+    -------
+    Element
+        xml reference to gmw object
+
+    Raises
+    ------
+    ValueError
+        if bro_id is invalid.
+    """
+
+    if not bro_id.startswith("GMW"):
+        raise ValueError("can only get metadata if bro id starts with GMW")
+
+    ns = {
+        "dsgmw": "http://www.broservices.nl/xsd/dsgmw/1.1",
+        "gmwcommon": "http://www.broservices.nl/xsd/gmwcommon/1.1",
+        "gml": "http://www.opengis.net/gml/3.2",
+    }
+
+    url = f"https://publiek.broservices.nl/gm/gmw/v1/objects/{bro_id}"
+    req = requests.get(url)
+
+    # read results
+    tree = xml.etree.ElementTree.fromstring(req.text)
+
+    gmws = tree.findall(".//dsgmw:GMW_PO", ns)
+    if len(gmws) != 1:
+        max_retries = rcParams["bro"]["max_retries"]
+        val_ind = req.text.find("valid")
+        valid = req.text[(val_ind + 9) : (val_ind + 14)]
+        if valid == "false" and retries < max_retries:
+            logger.debug(
+                f"got invalid response for {bro_id}, trying again {retries + 1}/{max_retries}"
+            )
+            return _get_gmw_from_bro_id(bro_id, retries=retries + 1)
+        elif valid == "false":
+            raise Exception(
+                f"got invalid response for {bro_id} after trying {retries} times"
+            )
+        raise (Exception("Only one gmw supported"))
+    gmw = gmws[0]
+
+    return gmw
+
+
+def get_tube_nrs_from_gmw(bro_id):
+    """returns all tube numbers from a groundwater monitoring well (gmw)
+
+    Parameters
+    ----------
+    bro_id : str
+        bro id of groundwater monitoring well e.g. 'GMW000000036287'.
+
+    Returns
+    -------
+    list of int
+        tube numbers
+    """
+    ns = {
+        "dsgmw": "http://www.broservices.nl/xsd/dsgmw/1.1",
+        "gmwcommon": "http://www.broservices.nl/xsd/gmwcommon/1.1",
+        "gml": "http://www.opengis.net/gml/3.2",
+    }
+
+    gmw = _get_gmw_from_bro_id(bro_id)
+
+    # get tube nrs
+    tube_numbers = [
+        int(tube.text)
+        for tube in gmw.findall("dsgmw:monitoringTube/dsgmw:tubeNumber", ns)
+    ]
+
+    return tube_numbers
+
+
 def get_metadata_from_gmw(bro_id, tube_nr):
     """get selection of metadata for a groundwater monitoring well.
     coordinates, ground_level, tube_top and tube screen
 
-
     Parameters
     ----------
     bro_id : str
@@ -461,34 +552,34 @@ def get_metadata_from_gmw(bro_id, tube_nr):
         dictionary with metadata.
 
     """
+    if not isinstance(tube_nr, int):
+        raise TypeError(f"expected integer got {type(tube_nr)}")
+
     ns = {
         "dsgmw": "http://www.broservices.nl/xsd/dsgmw/1.1",
         "gmwcommon": "http://www.broservices.nl/xsd/gmwcommon/1.1",
         "gml": "http://www.opengis.net/gml/3.2",
     }
 
-    if not bro_id.startswith("GMW"):
-        raise ValueError("can only get metadata if bro id starts with GMW")
+    gmw = _get_gmw_from_bro_id(bro_id)
 
-    if not isinstance(tube_nr, int):
-        raise TypeError(f"expected integer got {type(tube_nr)}")
-
-    url = f"https://publiek.broservices.nl/gm/gmw/v1/objects/{bro_id}"
-    req = requests.get(url)
-
-    # read results
-    tree = xml.etree.ElementTree.fromstring(req.text)
-
-    gmws = tree.findall(".//dsgmw:GMW_PO", ns)
-    if len(gmws) != 1:
-        raise (Exception("Only one gmw supported"))
-    gmw = gmws[0]
-
-    meta = {"monitoring_well": bro_id, "tube_nr": tube_nr, "source": "BRO"}
+    meta = {"location": bro_id, "tube_nr": tube_nr, "source": "BRO"}
 
     # x and y
-    xy = gmw.find("dsgmw:deliveredLocation//gmwcommon:location//gml:pos", ns)
-    meta["x"], meta["y"] = [float(val) for val in xy.text.split()]
+    xy_elem = gmw.find("dsgmw:deliveredLocation//gmwcommon:location//gml:pos", ns)
+    xy = [float(val) for val in xy_elem.text.split()]
+
+    # convert crs
+    srsname = gmw.find("dsgmw:deliveredLocation//gmwcommon:location", ns).attrib[
+        "srsName"
+    ]
+    epsg_gwm = int(srsname.split(":")[-1])
+    proj_from = Proj(f"EPSG:{epsg_gwm}")
+    proj_to = Proj(EPSG_28992)
+    transformer = Transformer.from_proj(proj_from, proj_to)
+    xy = transformer.transform(xy[0], xy[1])
+
+    meta["x"], meta["y"] = xy
 
     # ground_level
     vert_pos = gmw.find("dsgmw:deliveredVerticalPosition", ns)
@@ -505,9 +596,13 @@ def get_metadata_from_gmw(bro_id, tube_nr):
 
     # buis eigenschappen
     tubes = gmw.findall("dsgmw:monitoringTube", ns)
-    for tube in tubes:
-        if int(tube.find("dsgmw:tubeNumber", ns).text) == tube_nr:
-            break
+    tube_nrs = [int(tube.find("dsgmw:tubeNumber", ns).text) for tube in tubes]
+    if tube_nr not in tube_nrs:
+        raise ValueError(
+            f"gmw {bro_id} has no tube_nr {tube_nr} please choose a tube_nr from"
+            f"{tube_nrs}"
+        )
+    tube = tubes[tube_nrs.index(tube_nr)]
 
     # tube_top
     mp = tube.find("dsgmw:tubeTopPosition", ns)
@@ -574,6 +669,13 @@ def get_obs_list_from_extent(
         DESCRIPTION.
 
     """
+
+    if only_metadata and not keep_all_obs:
+        logger.error(
+            "you will get an empty ObsCollection with only_metadata is True and"
+            "keep_all_obs is False"
+        )
+
     url = "https://publiek.broservices.nl/gm/gmw/v1/characteristics/searches?"
 
     data = {}
@@ -586,18 +688,28 @@ def get_obs_list_from_extent(
             endDate = pd.to_datetime(tmax).strftime("%Y-%m-%d")
             data["registrationPeriod"]["endDate"] = endDate
 
-    transformer = Transformer.from_crs(epsg, 4326)
     data["area"] = {}
-    if extent is not None:
-        lat1, lon1 = transformer.transform(extent[0], extent[2])
-        lat2, lon2 = transformer.transform(extent[1], extent[3])
+    if epsg == 4326:
         data["area"]["boundingBox"] = {
-            "lowerCorner": {"lat": lat1, "lon": lon1},
-            "upperCorner": {"lat": lat2, "lon": lon2},
+            "lowerCorner": {"lat": extent[2], "lon": extent[0]},
+            "upperCorner": {"lat": extent[3], "lon": extent[1]},
         }
+    else:
+        transformer = Transformer.from_crs(epsg, 4326)
+        if extent is not None:
+            lat1, lon1 = transformer.transform(extent[0], extent[2])
+            lat2, lon2 = transformer.transform(extent[1], extent[3])
+            data["area"]["boundingBox"] = {
+                "lowerCorner": {"lat": lat1, "lon": lon1},
+                "upperCorner": {"lat": lat2, "lon": lon2},
+            }
     req = requests.post(url, json=data)
     if req.status_code > 200:
-        print(req.json()["errors"][0]["message"])
+        logger.error(
+            "could not get monitoring wells, your extent is probably too big."
+            "Try a smaller extent"
+        )
+        req.raise_for_status()
 
     # read results
     tree = xml.etree.ElementTree.fromstring(req.text)
@@ -615,7 +727,7 @@ def get_obs_list_from_extent(
         [gmw.text for gmw in tree.findall(".//dsgmw:GMW_C//brocom:broId", ns)]
     )
 
-    if len(gmws_ids) > 1000:
+    if len(gmws_ids) > 1000 and not ignore_max_obs:
         ans = input(
             f"You requested to download {len(gmws_ids)} observations, this can"
             "take a while. Are you sure you want to continue [Y/n]? "
@@ -628,11 +740,9 @@ def get_obs_list_from_extent(
         gmws = tree.findall(f'.//*[brocom:broId="{gmw_id}"]', ns)
         if len(gmws) < 1:
             raise RuntimeError("unexpected")
-        else:
-            gmw = gmws[0]
 
-        ntubes = int(gmw.find("dsgmw:numberOfMonitoringTubes", ns).text)
-        for tube_nr in range(1, ntubes + 1):
+        tube_nrs = get_tube_nrs_from_gmw(gmw_id)
+        for tube_nr in tube_nrs:
             o = ObsClass.from_bro(
                 gmw_id,
                 tube_nr=tube_nr,
@@ -641,7 +751,7 @@ def get_obs_list_from_extent(
                 only_metadata=only_metadata,
             )
             if o.empty:
-                logger.warning(
+                logger.debug(
                     f"no measurements found for gmw_id {gmw_id} and tube number"
                     f"{tube_nr}"
                 )
