@@ -3,36 +3,30 @@ import logging
 import os
 import xml.etree.ElementTree as etree
 from io import StringIO
+from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
-from lxml.etree import iterparse
 
-from ..observation import (
-    EvaporationObs,
-    GroundwaterObs,
-    MeteoObs,
-    PrecipitationObs,
-    WaterlvlObs,
-)
+from .. import observation
+from ..observation import Obs
 
 logger = logging.getLogger(__name__)
 
 
 def read_xml_fname(
-    fname,
-    ObsClass,
-    translate_dic=None,
-    low_memory=True,
-    locationIds=None,
-    filterdict=None,
-    return_events=True,
-    keep_flags=(0, 1),
-    return_df=False,
-    tags=("series", "header", "event"),
-    skip_errors=True,
-    remove_nan=False,
-    **kwargs,
+    fname: str,
+    ObsClass: Union[Obs, Dict[str, Obs]],
+    translate_dic: Optional[Dict[str, str]] = None,
+    low_memory: bool = True,
+    locationIds: Optional[List[str]] = None,
+    filterdict: Optional[Dict[str, List[str]]] = None,
+    return_events: bool = True,
+    keep_flags: Tuple[int] = (0, 1),
+    return_df: bool = False,
+    tags: Tuple[str] = ("series", "header", "event"),
+    remove_nan: bool = False,
+    **kwargs: dict,  # unused
 ):
     """Read an xml filename into a list of observations objects.
 
@@ -40,11 +34,11 @@ def read_xml_fname(
     ----------
     fname : str
         full path to file
-    ObsClass : type
+    ObsClass: Union[Obs, Dict[str, Obs]]
         class of the observations, e.g. GroundwaterObs or WaterlvlObs
     translate_dic : dic or None, optional
         translate names from fews. If None this default dictionary is used:
-        {'locationId': 'monitoring_well'}.
+        {'locationId': 'location'}.
     low_memory : bool, optional
         whether to use xml-parsing method with lower memory footprint,
         default is True
@@ -76,7 +70,7 @@ def read_xml_fname(
         list of timeseries stored in ObsClass objects
     """
     if translate_dic is None:
-        translate_dic = {"locationId": "monitoring_well"}
+        translate_dic = {"locationId": "location"}
 
     if low_memory is True:
         obs_list = iterparse_pi_xml(
@@ -89,7 +83,6 @@ def read_xml_fname(
             keep_flags=keep_flags,
             return_df=return_df,
             tags=tags,
-            skip_errors=skip_errors,
         )
     else:
         tree = etree.parse(fname)
@@ -106,16 +99,15 @@ def read_xml_fname(
 
 
 def iterparse_pi_xml(
-    fname,
-    ObsClass,
-    translate_dic=None,
-    filterdict=None,
-    locationIds=None,
-    return_events=True,
-    keep_flags=(0, 1),
-    return_df=False,
-    tags=("series", "header", "event"),
-    skip_errors=False,
+    fname: str,
+    ObsClass: Union[Obs, Dict[str, Obs]],
+    translate_dic: Optional[Dict[str, str]] = None,
+    filterdict: Optional[Dict[str, List[str]]] = None,
+    locationIds: Optional[List[str]] = None,
+    return_events: bool = True,
+    keep_flags: Tuple[int] = (0, 1),
+    return_df: bool = False,
+    tags: Tuple[str] = ("series", "header", "event"),
 ):
     """Read a FEWS XML-file with measurements, memory efficient.
 
@@ -123,11 +115,11 @@ def iterparse_pi_xml(
     ----------
     fname : str
         full path to file
-    ObsClass : type
+    ObsClass: Union[Obs, Dict[str, Obs]],
         class of the observations, e.g. GroundwaterObs or WaterlvlObs
     translate_dic : dic or None, optional
         translate names from fews. If None this default dictionary is used:
-        {'locationId': 'monitoring_well'}.
+        {'locationId': 'location'}.
     locationIds : tuple or list of str, optional
         list of locationId's to read from XML file, others are skipped.
         If None (default) all locations are read.
@@ -155,8 +147,10 @@ def iterparse_pi_xml(
     obs_list : list of pandas Series
         list of timeseries if 'return_df' is False
     """
+    from lxml.etree import iterparse
+
     if translate_dic is None:
-        translate_dic = {"locationId": "monitoring_well"}
+        translate_dic = {"locationId": "location"}
 
     tags = ["{{http://www.wldelft.nl/fews/PI}}{}".format(tag) for tag in tags]
 
@@ -184,8 +178,7 @@ def iterparse_pi_xml(
                     loc = h_attr.text
                     if loc not in locationIds:
                         element.clear()
-                        logger.info(
-                            f" ... skipping '{loc}', not in locationIds")
+                        logger.info(f" ... skipping '{loc}', not in locationIds")
                         continue
 
                 if filterdict is not None:
@@ -283,13 +276,13 @@ def iterparse_pi_xml(
 
 
 def read_xmlstring(
-    xmlstring,
-    ObsClass,
-    translate_dic=None,
-    filterdict=None,
-    locationIds=None,
-    low_memory=True,
-    remove_nan=False,
+    xmlstring: str,
+    ObsClass: Union[Obs, Dict[str, Obs]],
+    translate_dic: Optional[Dict[str, str]] = None,
+    filterdict: Optional[Dict[str, List[str]]] = None,
+    locationIds: Optional[List[str]] = None,
+    low_memory: bool = True,
+    remove_nan: bool = False,
 ):
     """Read xmlstring into an list of Obs objects. Xmlstrings are usually
     obtained using a fews api.
@@ -298,11 +291,11 @@ def read_xmlstring(
     ----------
     xmlstring : str
         xml string to be parsed. Typically from a fews api.
-    ObsClass : type
+    ObsClass: Union[Obs, Dict[str, Obs]]
         class of the observations, e.g. GroundwaterObs or WaterlvlObs
     translate_dic : dic or None, optional
         translate names from fews. If None this default dictionary is used:
-        {'locationId': 'monitoring_well'}.
+        {'locationId': 'location'}.
     locationIds : tuple or list of str, optional
         list of locationId's to read from XML file, others are skipped.
         If None (default) all locations are read.
@@ -319,7 +312,7 @@ def read_xmlstring(
         list of timeseries stored in ObsClass objects
     """
     if translate_dic is None:
-        translate_dic = {"locationId": "monitoring_well"}
+        translate_dic = {"locationId": "location"}
 
     if low_memory:
         obs_list = iterparse_pi_xml(
@@ -343,11 +336,11 @@ def read_xmlstring(
 
 
 def read_xml_root(
-    root,
-    ObsClass,
-    translate_dic=None,
-    locationIds=None,
-    remove_nan=False,
+    root: etree.Element,
+    ObsClass: Union[Obs, Dict[str, Obs]],
+    translate_dic: Dict[str, str] = None,
+    locationIds: List[str] = None,
+    remove_nan: bool = False,
 ):
     """Read a FEWS XML-file with measurements, return list of ObsClass objects.
 
@@ -355,11 +348,11 @@ def read_xml_root(
     ----------
     root : xml.etree.ElementTree.Element
         root element of a fews xml
-    ObsClass : type
+    ObsClass: Union[Obs, Dict[str, Obs]],
         class of the observations, e.g. GroundwaterObs or WaterlvlObs
     translate_dic : dic or None, optional
         translate names from fews. If None this default dictionary is used:
-        {'locationId': 'monitoring_well'}.
+        {'locationId': 'location'}.
     locationIds : tuple or list of str, optional
         list of locationId's to read from XML file, others are skipped.
         If None (default) all locations are read.
@@ -373,7 +366,7 @@ def read_xml_root(
         list of timeseries stored in ObsClass objects
     """
     if translate_dic is None:
-        translate_dic = {"locationId": "monitoring_well"}
+        translate_dic = {"locationId": "location"}
 
     obs_list = []
     for item in root:
@@ -402,15 +395,17 @@ def read_xml_root(
                 [d + " " + t for d, t in zip(date, time)], errors="coerce"
             )
             ts = pd.DataFrame(events, index=index)
-            ts.loc[:, "value"] = ts.loc[:, "value"].astype(float)
 
-            if remove_nan and (not ts.empty):
-                ts.dropna(subset=["value"], inplace=True)
-                header["unit"] = "m NAP"
+            if not ts.empty:
+                ts["value"] = ts["value"].astype(float)
+
+                if remove_nan:
+                    ts.dropna(subset=["value"], inplace=True)
+                    header["unit"] = "m NAP"
 
             o, header = _obs_from_meta(ts, header, translate_dic, ObsClass)
             if locationIds is not None:
-                if header["monitoring_well"] in locationIds:
+                if header["location"] in locationIds:
                     obs_list.append(o)
             else:
                 obs_list.append(o)
@@ -418,7 +413,12 @@ def read_xml_root(
     return obs_list
 
 
-def _obs_from_meta(ts, header, translate_dic, ObsClass):
+def _obs_from_meta(
+    ts: pd.DataFrame,
+    header: Dict[str, str],
+    translate_dic: Dict[str, str],
+    ObsClass: Union[Obs, Dict[str, Obs]],
+):
     """Internal function to convert timeseries and header into Obs objects.
 
     Parameters
@@ -429,7 +429,7 @@ def _obs_from_meta(ts, header, translate_dic, ObsClass):
         metadata.
     translate_dic : dictionary
         translate dictionary.
-    ObsClass : type
+    ObsClass: Union[Obs, Dict[str, Obs]],
         class of the observations, e.g. GroundwaterObs or WaterlvlObs
 
     Returns
@@ -462,29 +462,37 @@ def _obs_from_meta(ts, header, translate_dic, ObsClass):
         metadata_available = True
 
     if "parameterId" in header:
-        pid = header["parameterId"]
-        name = header["monitoring_well"] + "_" + pid
+        parid = header["parameterId"]
+        name = header["location"] + "_" + parid
     else:
-        name = header["monitoring_well"]
+        name = header["location"]
 
-    if ObsClass in (WaterlvlObs,):
-        o = ObsClass(
+    if isinstance(ObsClass, dict):
+        if parid in ObsClass.keys():
+            ObsC = ObsClass[parid]
+        else:
+            ObsC = Obs
+    else:
+        ObsC = ObsClass
+
+    if ObsC in (observation.WaterlvlObs,):
+        o = ObsC(
             ts,
             x=x,
             y=y,
             unit=unit,
             meta=header,
             name=name,
-            monitoring_well=header["monitoring_well"],
+            location=header["location"],
             metadata_available=metadata_available,
             source="FEWS",
         )
-    elif ObsClass in (GroundwaterObs,):
+    elif ObsC in (observation.GroundwaterObs,):
         if "z" in header.keys():
             z = float(header["z"])
         else:
             z = np.nan
-        o = ObsClass(
+        o = ObsC(
             ts,
             x=x,
             y=y,
@@ -492,29 +500,32 @@ def _obs_from_meta(ts, header, translate_dic, ObsClass):
             unit=unit,
             meta=header,
             name=name,
-            monitoring_well=header["monitoring_well"],
+            location=header["location"],
             metadata_available=metadata_available,
             source="FEWS",
         )
-    elif ObsClass in (MeteoObs, PrecipitationObs, EvaporationObs):
-        o = ObsClass(
+    elif ObsC in (
+        observation.MeteoObs,
+        observation.PrecipitationObs,
+        observation.EvaporationObs,
+    ):
+        o = ObsC(
             ts,
             x=x,
             y=y,
             unit=unit,
             meta=header,
             name=name,
-            meteo_var=pid,
+            meteo_var=parid,
             source="FEWS",
         )
     else:
-        o = ObsClass(ts, x=x, y=y, unit=unit, meta=header,
-                     name=name, source="FEWS")
+        o = ObsC(ts, x=x, y=y, unit=unit, meta=header, name=name, source="FEWS")
 
     return o, header
 
 
-def write_pi_xml(obs_coll, fname, timezone=1.0, version="1.24"):
+def write_pi_xml(obs_coll, fname: str, timezone: float = 1.0, version: str = "1.24"):
     """Write TimeSeries object to PI-XML file.
 
     Parameters
@@ -581,8 +592,7 @@ def write_pi_xml(obs_coll, fname, timezone=1.0, version="1.24"):
                         tag=htag, date=hdate, time=htime
                     )
                 elif htag.endswith("timeStep"):
-                    hline = '<{tag} unit="{unit}"/>\n'.format(
-                        tag=htag, unit=hval)
+                    hline = '<{tag} unit="{unit}"/>\n'.format(tag=htag, unit=hval)
                 else:
                     hline = paramline.format(tag=htag, param=hval)
                 hlines.append(3 * "\t" + hline)
@@ -615,15 +625,15 @@ def write_pi_xml(obs_coll, fname, timezone=1.0, version="1.24"):
 
 
 def read_xml_filelist(
-    fnames,
-    ObsClass,
-    directory=None,
-    locations=None,
-    translate_dic=None,
-    filterdict=None,
-    remove_nan=False,
-    low_memory=True,
-    **kwargs,
+    fnames: List[str],
+    ObsClass: Union[Obs, Dict[str, Obs]],
+    directory: Optional[str] = None,
+    locations: Optional[List[str]] = None,
+    translate_dic: Optional[Dict[str, str]] = None,
+    filterdict: Optional[Dict[str, List[str]]] = None,
+    remove_nan: bool = False,
+    low_memory: bool = True,
+    **kwargs: dict,
 ):
     """Read a list of xml files into a list of observation objects.
 
@@ -631,7 +641,7 @@ def read_xml_filelist(
     ----------
     fnames : TYPE
         DESCRIPTION.
-    ObsClass : type
+    ObsClass: Union[Obs, Dict[str, Obs]]
         class of the observations, e.g. GroundwaterObs or WaterlvlObs
     directory : TYPE, optional
         DESCRIPTION. The default is None.
@@ -640,7 +650,7 @@ def read_xml_filelist(
         If None (default) all locations are read.
     translate_dic : dic or None, optional
         translate names from fews. If None this default dictionary is used:
-        {'locationId': 'monitoring_well'}.
+        {'locationId': 'location'}.
     filterdict : dict, optional
         dictionary with tag name to apply filter to as keys, and list of
         accepted names as dictionary values to keep in final result,
@@ -658,13 +668,13 @@ def read_xml_filelist(
         list of timeseries stored in ObsClass objects
     """
     if translate_dic is None:
-        translate_dic = {"locationId": "monitoring_well"}
+        translate_dic = {"locationId": "location"}
 
     obs_list = []
     nfiles = len(fnames)
     for j, ixml in enumerate(fnames):
         # print message
-        logger.info(f"{j+1}/{nfiles} read {ixml}")
+        logger.info(f"{j + 1}/{nfiles} read {ixml}")
 
         # join directory to filename if provided
         if directory is None:
@@ -680,7 +690,28 @@ def read_xml_filelist(
             filterdict=filterdict,
             low_memory=low_memory,
             locationIds=locations,
+            remove_nan=remove_nan,
             **kwargs,
         )
 
     return obs_list
+
+
+def get_fews_pid(name: str) -> Dict[str, Obs]:
+    """Get matching ParameterId's and HydroPandas Observation Classes
+
+    Parameters
+    ----------
+    name : str
+        Waterboard name
+
+    Returns
+    -------
+    Dict[str, Obs]
+        Dictonary with ParameterId and the resulting Observation Class
+    """
+    from ..data.fews_parameterid import pid
+
+    pid_sel = pid[name.lower()]
+
+    return {key: getattr(observation, value) for (key, value) in pid_sel.items()}
